﻿using System;
using System.Reflection;
using System.Windows.Input;
using Syncfusion.ListView.XForms;
using Xamarin.Forms;

namespace TestListView
{
    public class CustomerCard
    {
        public long CustomerID { get; set; }
        public long CustomerCardID { get; set; }
        public DateTime ExpiryDate { get; set; }
        public string Last4Digits { get; set; }
        public string CardHolderName { get; set; }
        public int CustomerCardStatusID { get; set; }
        public bool IsSelected { get; set; }
    }

    public class BehaviorBase<T> : Behavior<T> where T : BindableObject
    {
        public T AssociatedObject { get; private set; }

        protected override void OnAttachedTo(T bindable)
        {
            base.OnAttachedTo(bindable);
            AssociatedObject = bindable;

            if (bindable.BindingContext != null)
            {
                BindingContext = bindable.BindingContext;
            }

            bindable.BindingContextChanged += OnBindingContextChanged;
        }

        protected override void OnDetachingFrom(T bindable)
        {
            base.OnDetachingFrom(bindable);
            bindable.BindingContextChanged -= OnBindingContextChanged;
            AssociatedObject = null;
        }

        void OnBindingContextChanged(object sender, EventArgs e)
        {
            OnBindingContextChanged();
        }

        protected override void OnBindingContextChanged()
        {
            base.OnBindingContextChanged();
            BindingContext = AssociatedObject.BindingContext;
        }
    }

    public class EventToCommandBehavior<T> : BehaviorBase<T> where T : BindableObject
    {
        Delegate _eventHandler;

        public static readonly BindableProperty EventNameProperty =
     BindableProperty.Create("EventName", typeof(string),
       typeof(EventToCommandBehavior<T>), null,
       propertyChanged: OnEventNameChanged);

        public static readonly BindableProperty CommandProperty =
          BindableProperty.Create("Command", typeof(ICommand),
            typeof(EventToCommandBehavior<T>), null);

        public static readonly BindableProperty
                   CommandParameterProperty =
          BindableProperty.Create("CommandParameter",
          typeof(object),
          typeof(EventToCommandBehavior<T>), null);

        public static readonly BindableProperty
        InputConverterProperty =
          BindableProperty.Create("Converter",
            typeof(IValueConverter),
            typeof(EventToCommandBehavior<T>), null);


        public string EventName
        {
            get { return (string)GetValue(EventNameProperty); }
            set { SetValue(EventNameProperty, value); }
        }

        public ICommand Command
        {
            get { return (ICommand)GetValue(CommandProperty); }
            set { SetValue(CommandProperty, value); }
        }

        public object CommandParameter
        {
            get { return GetValue(CommandParameterProperty); }
            set { SetValue(CommandParameterProperty, value); }
        }

        public IValueConverter Converter
        {
            get
            {
                return (IValueConverter)GetValue(
                    InputConverterProperty);
            }
            set { SetValue(InputConverterProperty, value); }
        }

        protected override void OnAttachedTo(
          BindableObject bindable)
        {
            base.OnAttachedTo(bindable);
            RegisterEvent(EventName);
        }

        protected override void OnDetachingFrom(
                 BindableObject bindable)
        {
            DeregisterEvent(EventName);
            base.OnDetachingFrom(bindable);
        }

        void RegisterEvent(string name)
        {
            if (string.IsNullOrWhiteSpace(name))
            {
                return;
            }

            EventInfo eventInfo = AssociatedObject.GetType()
                   .GetRuntimeEvent(name);
            if (eventInfo == null)
            {
                throw new ArgumentException(
                     "$EventToCommandBehavior: Can't register the '{EventName}' event.");
            }
            MethodInfo methodInfo = typeof(EventToCommandBehavior<T>)
                  .GetTypeInfo().GetDeclaredMethod("OnEvent");
            _eventHandler = methodInfo.CreateDelegate(
                eventInfo.EventHandlerType, this);
            eventInfo.AddEventHandler(AssociatedObject, _eventHandler);
        }

        void DeregisterEvent(string name)
        {
            if (string.IsNullOrWhiteSpace(name))
            {
                return;
            }

            if (_eventHandler == null)
            {
                return;
            }
            EventInfo eventInfo = AssociatedObject.GetType()
                    .GetRuntimeEvent(name);
            if (eventInfo == null)
            {
                throw new ArgumentException("$EventToCommandBehavior:Can't de-register the '{ EventName }' event.");
            }
            eventInfo.RemoveEventHandler(
                 AssociatedObject, _eventHandler);
            _eventHandler = null;
        }

        void OnEvent(object sender, object eventArgs)
        {
            if (Command == null)
            {
                return;
            }

            object resolvedParameter;

            if (CommandParameter != null)
            {
                resolvedParameter = CommandParameter;
            }
            else if (Converter != null)
            {
                resolvedParameter = Converter.Convert(
                         eventArgs, typeof(object), null, null);
            }
            else if (eventArgs is SelectedItemChangedEventArgs)
            {
                resolvedParameter =
                    ((SelectedItemChangedEventArgs)eventArgs)
                      .SelectedItem;
            }
            else
            {
                resolvedParameter = eventArgs;
            }

            if (Command.CanExecute(resolvedParameter))
            {
                Command.Execute(resolvedParameter);
            }
        }

        static void OnEventNameChanged(
             BindableObject bindable,
             object oldValue,
             object newValue)
        {
            var behavior = (EventToCommandBehavior<T>)bindable;
            if (behavior.AssociatedObject == null)
            {
                return;
            }

            string oldEventName = (string)oldValue;
            string newEventName = (string)newValue;

            behavior.DeregisterEvent(oldEventName);
            behavior.RegisterEvent(newEventName);
        }
    }

    public class SyncFusionListItemBehavior : EventToCommandBehavior<SfListView>
    {

    }
}